FlexBar2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0", "AceDebug-2.0")
FlexBar2.OO = AceLibrary("AceOO-2.0");
FlexBar2.Waterfall = AceLibrary("Waterfall-1.0");

FlexBar2:RegisterDB("FlexBar2_DB");
FlexBar2:RegisterDefaults('profile', {
	Buttons = {},
});

FlexBar2:RegisterChatCommand({"/flexbar", "/flexbar2", "/fb", "/fb2"}, 
{
    type = 'execute',
    desc = "Show configuration screen",
    func = function()
        FlexBar2.Waterfall:Open("FlexBar2");
    end
});

function FlexBar2:OnInitialize()
    self:SetDebugging(true);
	self.ModuleOnEnableList = {};
	self.ButtonList = {};
	self.ButtonMixinList = {};
	self.GroupMixinList = {};
	self.ButtonActivatorList = {};
	self.GroupActivatorList = {};
	self.ExtraOptionsTableList = {};
	self.Buttons = {};
	self:Debug("Searching for modules");
    for Name, Module in self:IterateModules() do
		self:Debug("     Found Module: " .. Name);
		if(type(Module.ButtonMixin) == "table") then
			self.ButtonMixinList[Name] = Module.ButtonMixin
			self:Debug("          Found ButtonMixin");
			if(type(Module.ButtonMixin.Activate) == "function") then
				self:Debug("               Found Activator");
				table.insert(FlexBar2.ButtonActivatorList, Module.ButtonMixin.Activate);
			end
		end
		if(type(Module.GroupMixin) == "table") then
			self:Debug("          Found GroupMixin: ");
			self.GroupMixinList[Name] = Module.GroupMixin;
			if(type(Module.GroupMixin.Activate) == "function") then
				self:Debug("               Found Activator");
				table.insert(self.GroupActivatorList, Module.GroupMixin.Activate);
			end
		end
		if(type(Module.OnEnable) == "function") then
			self:Debug("          Found OnEnable handler");
			self.ModuleOnEnableList[Name] = Module.OnEnable;
		end
		if(type(Module.AceOptions) == "table") then
			self:Debug("          Found AceOptions");
			table.insert(self.ExtraOptionsTableList, Module.AceOptions);
		end
	end
	
	self.ButtonSuperClass = self.OO.Class(unpack(self.ButtonMixinList));
		function self.ButtonSuperClass.prototype:init(Name, ...)
			FlexBar2.ButtonSuperClass.super.prototype.init(self);
			FlexBar2:Debug("          Initializing");
			if(type(Name) ~= "string") then
				FlexBar2:Debug("               Failed, invalid name");
				return false;
			end
			self.InfoTable = FlexBar2.db.profile.Buttons[Name];
			FlexBar2.Buttons[Name] = self;
			table.insert(FlexBar2.ButtonList, Name);
			for _, Func in ipairs(FlexBar2.ButtonActivatorList) do
				Func(self, ...);
			end
		end
	self.GroupSuperClass = self.OO.Class(unpack(self.GroupMixinList));
		function self.GroupSuperClass.prototype:init(...)
			FlexBar2:Debug("          Initializing");
			FlexBar2.GroupSuperClass.super.prototype.init(self);
			for _, Func in ipairs(FlexBar2.GroupActivatorList) do
				Func(self, ...);
			end
		end
end

function FlexBar2:Generate(Name)
	if(self.Buttons[Name] ~= nil) then
		self:Debug("          Error: Button already exists, aborting");
		return false;
	end
	self.ButtonSuperClass:new(Name);
	return true;
end

function FlexBar2:OnEnable()
	self:Debug("Setting up Waterfall");
	self.AceSubOptions = {};
	self.AceOptions = {
		type = 'group',
		desc = "FlexBar2, Create an unlimited amount of *very* customizable buttons",
		args = self.AceSubOptions,
	};
	self.Waterfall:Register('FlexBar2',  'aceOptions', self.AceOptions, 'FlexBar2','FlexBar2')
		local buttonctrl_CreateSave;
	local buttonctrl_EditSave;
	local buttonctrl_EditSwitch = false;
	local function BuildWaterfallTree(Id)
		if not Id then return ""; end
		if(Id == "gnrlset") then
			
		elseif(Id == "buttonctrl") then
			if(buttonctrl_EditSwitch == false) then
				-- Create Button --
				Waterfall:AddControl(
				"type", "label",
				"text", "Create Button: ");
				
				Waterfall:AddControl(
				"type","textbox",
				"width",200,
				"noNewLine", true,
				"setFunc", FlexBar2.Generate,
				"setArg1", FlexBar2,
				"changedFunc", function(ButtonName) buttonctrl_CreateSave = ButtonName end);
				
				waterfall:AddControl(
				"type","button",
				"text","Create",
				"noNewLine",true,
				"width",25,
				"causeRefresh",true,
				"execFunc",FlexBar2.Generate,
				"execArg1", FlexBar2,
				"execArg2", buttonctrl_CreateSave);
				
				-- Edit Button --
				Waterfall:AddControl(
				"type", "label",
				"text", "Edit Button: ");
				
				Waterfall:AddControl(
				"type","textbox",
				"width",200,
				"noNewLine", true,
				"setFunc", function(ButtonName) buttonctrl_EditSave = ButtonName end,
				"changedFunc", function(ButtonName) buttonctrl_EditSwitch = ButtonName end);
				
				waterfall:AddControl(
				"type","button",
				"text","Edit",
				"noNewLine",true,
				"width",25,
				"execFunc", function(ButtonName) buttonctrl_EditSave = ButtonName end,
				"execArg1", buttonctrl_EditSwitch);
			else
				-- Delete Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Delete Button",
				"width", 25,
				"execFunc", FlexBar2.Delete,
				"execArg1", buttonctrl_EditSave);
				
				-- Save Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Delete Button",
				"width", 25,
				"execFunc", function() end);
				
				-- Cancel Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Delete Button",
				"width", 25,
				"execFunc", function() buttonctrl_EditSave = false; end,
				"execArg1");
			end
		elseif(Id == "groupctrl") then
			
		end
	end
	self:Debug("Generating buttons");
	for Name, InfoTable in pairs(self.db.profile.Buttons) do
		if(InfoTable.Generate == true) then
			self:Debug("     Generating " .. Name);
			self:Generate(Name);
		end
	end
	for _, Func in ipairs(self.ModuleOnEnableList) do
		Func(self);
	end
end

function FlexBar2:OnDisable()
    
end

function FlexBar2:OnModuleCreated(Name, ref)

end
